@@ -2,7 +2,7 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * dcraw_api.cc - API for DCRaw
- * Copyright 2004-2011 by Udi Fuchs
+ * Copyright 2004-2013 by Udi Fuchs
  *
  * based on dcraw by Dave Coffin
  * http://www.cybercom.net/~dcoffin/
@@ -34,10 +34,8 @@
 #define FORC3 FORC(3)
 #define FORC4 FORC(4)
 #define FORCC FORC(colors)
-#define FC(filters,row,col) \
-    (filters >> ((((row) << 1 & 14) + ((col) & 1)) << 1) & 3)
 extern "C" {
-    int fc_INDI(const unsigned filters, const int row, const int col);
+    int fcol_INDI(const unsigned filters, const int row, const int col);
     void wavelet_denoise_INDI(gushort(*image)[4], const int black,
                               const int iheight, const int iwidth, const int height, const int width,
                               const int colors, const int shrink, const float pre_mul[4],
@@ -93,6 +91,9 @@ extern "C" {
             return DCRAW_OPEN_ERROR;
         }
         d->identify();
+#ifndef UFRAW_X_TRANS
+        if (d->filters == 2) d->is_raw = 0;
+#endif
         /* We first check if dcraw recognizes the file, this is equivalent
          * to 'dcraw -i' succeeding */
         if (!d->make[0]) {
@@ -205,6 +206,7 @@ extern "C" {
         g_free(d->messageBuffer);
         d->messageBuffer = NULL;
         d->lastStatus = DCRAW_SUCCESS;
+        d->raw_image = 0;
         if (setjmp(d->failure)) {
             d->dcraw_message(DCRAW_ERROR, _("Fatal internal error\n"));
             h->message = d->messageBuffer;
@@ -213,8 +215,6 @@ extern "C" {
         }
         h->raw.height = d->iheight = (h->height + h->shrink) >> h->shrink;
         h->raw.width = d->iwidth = (h->width + h->shrink) >> h->shrink;
-        h->raw.image = d->image = g_new0(dcraw_image_type, d->iheight * d->iwidth
-                                         + d->meta_length);
         d->meta_data = (char *)(d->image + d->iheight * d->iwidth);
         /* copied from the end of dcraw's identify() */
         if (d->filters && d->colors == 3) {
@@ -223,17 +223,34 @@ extern "C" {
         }
         h->raw.colors = d->colors;
         h->fourColorFilters = d->filters;
+        if (d->filters) {
+            d->raw_image = (ushort *) g_malloc((d->raw_height + 7) * d->raw_width * 2);
+        } else {
+            h->raw.image = d->image = g_new0(dcraw_image_type, d->iheight * d->iwidth
+                                             + d->meta_length);
+        }
         d->dcraw_message(DCRAW_VERBOSE, _("Loading %s %s image from %s ...\n"),
                          d->make, d->model, d->ifname_display);
         fseek(d->ifp, 0, SEEK_END);
         d->ifpSize = ftell(d->ifp);
         fseek(d->ifp, d->data_offset, SEEK_SET);
         (d->*d->load_raw)();
+        h->raw.height = d->iheight = (h->height + h->shrink) >> h->shrink;
+        h->raw.width = d->iwidth = (h->width + h->shrink) >> h->shrink;
+        if (d->raw_image) {
+            h->raw.image = d->image = g_new0(dcraw_image_type, d->iheight * d->iwidth
+                                             + d->meta_length);
+            d->crop_masked_pixels();
+            g_free(d->raw_image);
+        }
         if (!--d->data_error) d->lastStatus = DCRAW_ERROR;
         if (d->zero_is_bad) d->remove_zeroes();
         d->bad_pixels(NULL);
         if (d->is_foveon) {
-            d->foveon_interpolate();
+            if (d->load_raw == &DCRaw::foveon_dp_load_raw) {
+                for (i = 0; i < d->height * d->width * 4; i++)
+                    if ((short) d->image[0][i] < 0) d->image[0][i] = 0;
+            } else d->foveon_interpolate();
             h->raw.width = h->width = d->width;
             h->raw.height = h->height = d->height;
         }
@@ -335,21 +352,21 @@ extern "C" {
     }
 
     /*
-     * fc_INDI() optimizing wrapper.
-     * fc_sequence() cooks up the filter color sequence for a row knowing that
+     * fcol_INDI() optimizing wrapper.
+     * fcol_sequence() cooks up the filter color sequence for a row knowing that
      * it doesn't have to store more than 16 values. The result can be indexed
-     * by the column using fc_color() and that part must of course be inlined
+     * by the column using fcol_color() and that part must of course be inlined
      * for maximum performance. The inner loop for image processing should
      * always try to index the column and not the row in order to reduce the
      * data cache footprint.
      */
-    static unsigned fc_sequence(int filters, int row)
+    static unsigned fcol_sequence(int filters, int row)
     {
         unsigned sequence = 0;
         int c;
 
         for (c = 15; c >= 0; --c)
-            sequence = (sequence << 2) | fc_INDI(filters, row, c);
+            sequence = (sequence << 2) | fcol_INDI(filters, row, c);
         return sequence;
     }
 
@@ -357,7 +374,7 @@ extern "C" {
      * Note: smart compilers will inline anyway in most cases: the "inline"
      * below is a comment reminding not to make it an external function.
      */
-    static inline int fc_color(unsigned sequence, int col)
+    static inline int fcol_color(unsigned sequence, int col)
     {
         return (sequence >> ((col << 1) & 0x1f)) & 3;
     }
@@ -409,7 +426,7 @@ extern "C" {
         for (ri = 0; ri < scale; ++ri) {
             ibase = hh->raw.image + ((row * scale + ri) / 2) * hh->raw.width;
             for (ci = 0; ci < scale; ++ci) {
-                cl = fc_color(fseq[ri], col * scale + ci);
+                cl = fcol_color(fseq[ri], col * scale + ci);
                 sum[cl] += ibase[(col * scale + ci) / 2][cl];
                 ++count[cl];
             }
@@ -422,7 +439,7 @@ extern "C" {
                               int scale)
     {
         DCRaw *d = (DCRaw *)hh->dcraw;
-        int h, w, fujiWidth, r, c, ri, recombine, pixels, f4;
+        int h, w, fujiWidth, r, c, ri, recombine, f4;
         dcraw_image_type *ibase, *obase;
         unsigned *fseq;
         unsigned short *pixp;
@@ -430,7 +447,6 @@ extern "C" {
         g_free(d->messageBuffer);
         d->messageBuffer = NULL;
         d->lastStatus = DCRAW_SUCCESS;
-        pixels = hh->raw.width * hh->raw.height;
 
         recombine = (hh->colors == 3 && hh->raw.colors == 4);
         /* the last row/column will be skipped if input is incomplete */
@@ -448,12 +464,12 @@ extern "C" {
             f4 = hh->fourColorFilters;
 
 #ifdef _OPENMP
-#pragma omp parallel for schedule(static) private(r,ri,fseq,c,pixp)
+            #pragma omp parallel for schedule(static) private(r,ri,fseq,c,pixp)
 #endif
             for (r = 0; r < h; ++r) {
                 fseq = (unsigned*) g_malloc(scale * sizeof(unsigned));
                 for (ri = 0; ri < scale; ++ri)
-                    fseq[ri] = fc_sequence(f4, r + ri);
+                    fseq[ri] = fcol_sequence(f4, r + ri);
                 for (c = 0; c < w; ++c) {
                     pixp = f->image[r * w + c];
                     shrink_pixel(pixp, r, c, hh, fseq, scale);
@@ -468,7 +484,7 @@ extern "C" {
             f->image = (dcraw_image_type *)g_realloc(
                            f->image, h * w * sizeof(dcraw_image_type));
 #ifdef _OPENMP
-#pragma omp parallel for schedule(static) private(r,ibase,obase,c)
+            #pragma omp parallel for schedule(static) private(r,ibase,obase,c)
 #endif
             for (r = 0; r < h; ++r) {
                 ibase = hh->raw.image + r * hh->raw.width * scale;
@@ -535,10 +551,10 @@ extern "C" {
                     ciw = 0;
                 }
                 for (cl = 0; cl < image->colors; cl++) {
-                    iBuf[ri *w+ci ][cl] += image->image[r*wid+c][cl] * riw * ciw ;
-                    iBuf[ri *w+cii][cl] += image->image[r*wid+c][cl] * riw * ciiw;
-                    iBuf[rii*w+ci ][cl] += image->image[r*wid+c][cl] * riiw * ciw ;
-                    iBuf[rii*w+cii][cl] += image->image[r*wid+c][cl] * riiw * ciiw;
+                    iBuf[ri * w + ci ][cl] += image->image[r * wid + c][cl] * riw * ciw ;
+                    iBuf[ri * w + cii][cl] += image->image[r * wid + c][cl] * riw * ciiw;
+                    iBuf[rii * w + ci ][cl] += image->image[r * wid + c][cl] * riiw * ciw ;
+                    iBuf[rii * w + cii][cl] += image->image[r * wid + c][cl] * riiw * ciiw;
                 }
             }
         }
@@ -564,10 +580,10 @@ extern "C" {
             iBuf = g_new(dcraw_image_type, image->width * newdim);
             for (rc = row = 0; row < newdim; row++, rc += pixel_aspect) {
                 frac = rc - (c = (int)rc);
-                pix0 = pix1 = image->image[c*image->width];
+                pix0 = pix1 = image->image[c * image->width];
                 if (c + 1 < image->height) pix1 += image->width * 4;
                 for (col = 0; col < image->width; col++, pix0 += 4, pix1 += 4)
-                    FORCC iBuf[row*image->width+col][c] =
+                    FORCC iBuf[row * image->width + col][c] =
                         (guint16)(pix0[c] * (1 - frac) + pix1[c] * frac + 0.5);
             }
             image->height = newdim;
@@ -580,7 +596,7 @@ extern "C" {
                 if (c + 1 < image->width) pix1 += 4;
                 for (row = 0; row < image->height;
                         row++, pix0 += image->width * 4, pix1 += image->width * 4)
-                    FORCC iBuf[row*newdim+col][c] =
+                    FORCC iBuf[row * newdim + col][c] =
                         (guint16)(pix0[c] * (1 - frac) + pix1[c] * frac + 0.5);
             }
             image->width = newdim;
@@ -609,8 +625,8 @@ extern "C" {
             for (int c = 0; c < 4; c++)
                 h->post_mul[c] = 1.0;
         } else {
-            scale_colors_INDI(h->rgbMax - h->black, h->black, useCameraWB,
-                              h->cam_mul, h->raw.colors, h->post_mul, h->filters, d->white,
+            scale_colors_INDI(h->rgbMax, h->black, useCameraWB, h->cam_mul,
+                              h->colors, h->post_mul, h->filters, d->white,
                               d->ifname_display, d);
         }
         h->message = d->messageBuffer;
@@ -647,8 +663,8 @@ extern "C" {
             rgbWB[3] = rgbWB[1];
         if (dark) {
 #ifdef _OPENMP
-#pragma omp parallel for schedule(static) default(none) \
-  shared(h,dark,rgbWB)
+            #pragma omp parallel for schedule(static) default(none) \
+            shared(h,dark,rgbWB)
 #endif
             for (int i = 0; i < pixels; i++) {
                 int cc;
@@ -660,8 +676,8 @@ extern "C" {
             }
         } else {
 #ifdef _OPENMP
-#pragma omp parallel for schedule(static) default(none) \
-  shared(h,dark,rgbWB)
+            #pragma omp parallel for schedule(static) default(none) \
+            shared(h,dark,rgbWB)
 #endif
             for (int i = 0; i < pixels; i++) {
                 int cc;
@@ -677,13 +693,12 @@ extern "C" {
                                    int interpolation, int smoothing)
     {
         DCRaw *d = (DCRaw *)h->dcraw;
-        int fujiWidth, i, r, c, cl, pixels;
+        int fujiWidth, i, r, c, cl;
         unsigned ff, f4;
 
         g_free(d->messageBuffer);
         d->messageBuffer = NULL;
         d->lastStatus = DCRAW_SUCCESS;
-        pixels = h->raw.width * h->raw.height;
 
         f->width = h->width;
         f->height = h->height;
@@ -707,16 +722,16 @@ extern "C" {
 
         /* It might be better to report an error here: */
         /* (dcraw also forbids AHD for Fuji rotated images) */
-        if (interpolation == dcraw_ahd_interpolation && h->colors > 3)
+        if (interpolation == dcraw_ahd_interpolation && (h->colors > 3 || h->filters < 1000))
             interpolation = dcraw_vng_interpolation;
-        if (interpolation == dcraw_ppg_interpolation && h->colors > 3)
+        if (interpolation == dcraw_ppg_interpolation && (h->colors > 3 || h->filters < 1000))
             interpolation = dcraw_vng_interpolation;
         f4 = h->fourColorFilters;
         for (r = 0; r < h->height; r++)
             for (c = 0; c < h->width; c++) {
-                int cc = fc_INDI(f4, r, c);
-                f->image[r*f->width+c][fc_INDI(ff, r, c)] =
-                    h->raw.image[r/2 * h->raw.width + c/2][cc];
+                int cc = fcol_INDI(f4, r, c);
+                f->image[r * f->width + c][fcol_INDI(ff, r, c)] =
+                    h->raw.image[r / 2 * h->raw.width + c / 2][cc];
             }
         int smoothPasses = 1;
         if (interpolation == dcraw_bilinear_interpolation)
