@@ -2,7 +2,7 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * dcraw_indi.c - DCRaw functions made independent
- * Copyright 2004-2011 by Udi Fuchs
+ * Copyright 2004-2013 by Udi Fuchs
  *
  * based on dcraw by Dave Coffin
  * http://www.cybercom.net/~dcoffin/
@@ -69,7 +69,7 @@ extern const float d65_white[3];
 #define BAYER(row,col) \
     image[((row) >> shrink)*iwidth + ((col) >> shrink)][FC(row,col)]
 
-int CLASS fc_INDI(const unsigned filters, const int row, const int col)
+int CLASS fcol_INDI(const unsigned filters, const int row, const int col)
 {
     static const char filter[16][16] = {
         { 2, 1, 1, 3, 2, 3, 2, 0, 3, 2, 3, 0, 1, 2, 1, 0 },
@@ -89,12 +89,25 @@ int CLASS fc_INDI(const unsigned filters, const int row, const int col)
         { 2, 1, 3, 2, 3, 1, 2, 1, 0, 3, 0, 2, 0, 2, 0, 2 },
         { 0, 3, 1, 0, 0, 2, 0, 3, 2, 1, 3, 1, 1, 3, 1, 3 }
     };
+#ifdef UFRAW_X_TRANS
+    static const char filter2[6][6] = {
+        { 1, 1, 0, 1, 1, 2 },
+        { 1, 1, 2, 1, 1, 0 },
+        { 2, 0, 1, 0, 2, 1 },
+        { 1, 1, 2, 1, 1, 0 },
+        { 1, 1, 0, 1, 1, 2 },
+        { 0, 2, 1, 2, 0, 1 }
+    };
+#endif
 
-    if (filters != 1) return FC(row, col);
     /* Assume that we are handling the Leaf CatchLight with
      * top_margin = 8; left_margin = 18; */
-//  return filter[(row+top_margin) & 15][(col+left_margin) & 15];
-    return filter[(row+8) & 15][(col+18) & 15];
+//  if (filters == 1) return filter[(row+top_margin) & 15][(col+left_margin) & 15];
+    if (filters == 1) return filter[(row + 8) & 15][(col + 18) & 15];
+#ifdef UFRAW_X_TRANS
+    if (filters == 2) return filter2[(row + 6) % 6][(col + 6) % 6];
+#endif
+    return FC(row, col);
 }
 
 static void CLASS merror(void *ptr, char *where)
@@ -107,20 +120,22 @@ static void CLASS hat_transform(float *temp, float *base, int st, int size, int
 {
     int i;
     for (i = 0; i < sc; i++)
-        temp[i] = 2 * base[st*i] + base[st*(sc-i)] + base[st*(i+sc)];
+        temp[i] = 2 * base[st * i] + base[st * (sc - i)] + base[st * (i + sc)];
     for (; i + sc < size; i++)
-        temp[i] = 2 * base[st*i] + base[st*(i-sc)] + base[st*(i+sc)];
+        temp[i] = 2 * base[st * i] + base[st * (i - sc)] + base[st * (i + sc)];
     for (; i < size; i++)
-        temp[i] = 2 * base[st*i] + base[st*(i-sc)] + base[st*(2*size-2-(i+sc))];
+        temp[i] = 2 * base[st * i] + base[st * (i - sc)] + base[st * (2 * size - 2 - (i + sc))];
 }
 
 void CLASS wavelet_denoise_INDI(ushort(*image)[4], const int black,
-    const int iheight, const int iwidth, const int height, const int width,
-    const int colors, const int shrink, const float pre_mul[4],
-    const float threshold, const unsigned filters)
+                                const int iheight, const int iwidth,
+                                const int height, const int width,
+                                const int colors, const int shrink,
+                                const float pre_mul[4], const float threshold,
+                                const unsigned filters)
 {
-    float *fimg = 0, /* *temp,*/ thold, mul[2], avg, diff;
-    int /*scale=1,*/ size, lev, hpass, lpass, row, col, nc, c, i, wlast;
+    float *fimg = 0, thold, mul[2], avg, diff;
+    int size, lev, hpass, lpass, row, col, nc, c, i, wlast;
     ushort *window[4];
     static const float noise[] =
     { 0.8002, 0.2735, 0.1202, 0.0585, 0.0291, 0.0152, 0.0080, 0.0044 };
@@ -133,10 +148,17 @@ void CLASS wavelet_denoise_INDI(ushort(*image)[4], const int black,
     if ((nc = colors) == 3 && filters) nc++;
     progress(PROGRESS_WAVELET_DENOISE, -nc * 5);
 #ifdef _OPENMP
-#pragma omp parallel for				\
-  default(none)						\
-  shared(nc,image,size)					\
-  private(c,i,hpass,lev,lpass,row,col,thold,fimg,temp)
+#ifdef __sun			/* Fix bug #3205673 - NKBJ */
+    #pragma omp parallel for				\
+    default(none)					\
+    shared(nc,image,size,noise)				\
+    private(c,i,hpass,lev,lpass,row,col,thold,fimg,temp)
+#else
+    #pragma omp parallel for				\
+    default(none)					\
+    shared(nc,image,size)				\
+    private(c,i,hpass,lev,lpass,row,col,thold,fimg,temp)
+#endif
 #endif
     FORC(nc) {			/* denoise R,G1,B,G3 individually */
         fimg = (float *) malloc(size * 3 * sizeof * fimg);
@@ -148,45 +170,45 @@ void CLASS wavelet_denoise_INDI(ushort(*image)[4], const int black,
             for (row = 0; row < iheight; row++) {
                 hat_transform(temp, fimg + hpass + row * iwidth, 1, iwidth, 1 << lev);
                 for (col = 0; col < iwidth; col++)
-                    fimg[lpass + row*iwidth + col] = temp[col] * 0.25;
+                    fimg[lpass + row * iwidth + col] = temp[col] * 0.25;
             }
             for (col = 0; col < iwidth; col++) {
                 hat_transform(temp, fimg + lpass + col, iwidth, iheight, 1 << lev);
                 for (row = 0; row < iheight; row++)
-                    fimg[lpass + row*iwidth + col] = temp[row] * 0.25;
+                    fimg[lpass + row * iwidth + col] = temp[row] * 0.25;
             }
             thold = threshold * noise[lev];
             for (i = 0; i < size; i++) {
-                fimg[hpass+i] -= fimg[lpass+i];
-                if	(fimg[hpass+i] < -thold) fimg[hpass+i] += thold;
-                else if (fimg[hpass+i] >  thold) fimg[hpass+i] -= thold;
-                else	 fimg[hpass+i] = 0;
-                if (hpass) fimg[i] += fimg[hpass+i];
+                fimg[hpass + i] -= fimg[lpass + i];
+                if	(fimg[hpass + i] < -thold) fimg[hpass + i] += thold;
+                else if (fimg[hpass + i] >  thold) fimg[hpass + i] -= thold;
+                else	 fimg[hpass + i] = 0;
+                if (hpass) fimg[i] += fimg[hpass + i];
             }
             hpass = lpass;
         }
         for (i = 0; i < size; i++)
-            image[i][c] = CLIP(SQR(fimg[i] + fimg[lpass+i]) / 0x10000);
+            image[i][c] = CLIP(SQR(fimg[i] + fimg[lpass + i]) / 0x10000);
         free(fimg);
     }
     if (filters && colors == 3) {  /* pull G1 and G3 closer together */
         for (row = 0; row < 2; row++)
-            mul[row] = 0.125 * pre_mul[FC(row+1, 0) | 1] / pre_mul[FC(row, 0) | 1];
+            mul[row] = 0.125 * pre_mul[FC(row + 1, 0) | 1] / pre_mul[FC(row, 0) | 1];
         ushort window_mem[4][width];
         for (i = 0; i < 4; i++)
             window[i] = window_mem[i]; /*(ushort *) fimg + width*i;*/
         for (wlast = -1, row = 1; row < height - 1; row++) {
             while (wlast < row + 1) {
                 for (wlast++, i = 0; i < 4; i++)
-                    window[(i+3) & 3] = window[i];
+                    window[(i + 3) & 3] = window[i];
                 for (col = FC(wlast, 1) & 1; col < width; col += 2)
                     window[2][col] = BAYER(wlast, col);
             }
             thold = threshold / 512;
             for (col = (FC(row, 0) & 1) + 1; col < width - 1; col += 2) {
-                avg = (window[0][col-1] + window[0][col+1] +
-                window[2][col-1] + window[2][col+1] - black * 4)
-                * mul[row & 1] + (window[1][col] - black) * 0.5 + black;
+                avg = (window[0][col - 1] + window[0][col + 1] +
+                       window[2][col - 1] + window[2][col + 1] - black * 4)
+                      * mul[row & 1] + (window[1][col] - black) * 0.5 + black;
                 avg = avg < 0 ? 0 : sqrt(avg);
                 diff = sqrt(BAYER(row, col)) - avg;
                 if (diff < -thold) diff += thold;
@@ -198,15 +220,14 @@ void CLASS wavelet_denoise_INDI(ushort(*image)[4], const int black,
     }
 }
 
-void CLASS scale_colors_INDI(int maximum, const int black,
-    const int use_camera_wb, const float cam_mul[4], const int colors,
-    float pre_mul[4], const unsigned filters, /*const*/ ushort white[8][8],
-    const char *ifname_display, void *dcraw)
+void CLASS scale_colors_INDI(const int maximum, const int black,
+                             const int use_camera_wb, const float cam_mul[4], const int colors,
+                             float pre_mul[4], const unsigned filters, /*const*/ ushort white[8][8],
+                             const char *ifname_display, void *dcraw)
 {
-    unsigned /*bottom, right, size,*/ row, col, /*ur, uc, i, x, y,*/ c, sum[8];
-    int val, dark, sat;
-    double /*dsum[8],*/ dmin, dmax;
-    float scale_mul[4]/*, fr, fc*/;
+    unsigned row, col, c, sum[8];
+    int val;
+    double dmin, dmax;
 
     if (use_camera_wb && cam_mul[0] != -1) {
         memset(sum, 0, sizeof sum);
@@ -215,34 +236,31 @@ void CLASS scale_colors_INDI(int maximum, const int black,
                 c = FC(row, col);
                 if ((val = white[row][col] - black) > 0)
                     sum[c] += val;
-                sum[c+4]++;
+                sum[c + 4]++;
             }
         if (sum[0] && sum[1] && sum[2] && sum[3])
-            FORC4 pre_mul[c] = (float) sum[c+4] / sum[c];
+            FORC4 pre_mul[c] = (float) sum[c + 4] / sum[c];
         else if (cam_mul[0] && cam_mul[2])
             /* 'sizeof pre_mul' does not work because pre_mul is an argument (UF)*/
             memcpy(pre_mul, cam_mul, 4 * sizeof(float));
         else
             dcraw_message(dcraw, DCRAW_NO_CAMERA_WB,
-            _("%s: Cannot use camera white balance.\n"), ifname_display);
+                          _("%s: Cannot use camera white balance.\n"), ifname_display);
     } else {
         dcraw_message(dcraw, DCRAW_NO_CAMERA_WB,
-        _("%s: Cannot use camera white balance.\n"), ifname_display);
+                      _("%s: Cannot use camera white balance.\n"), ifname_display);
     }
     if (pre_mul[3] == 0) pre_mul[3] = colors < 4 ? pre_mul[1] : 1;
-    dark = black;
-    sat = maximum;
-    maximum -= black;
     for (dmin = DBL_MAX, dmax = c = 0; c < 4; c++) {
         if (dmin > pre_mul[c])
             dmin = pre_mul[c];
         if (dmax < pre_mul[c])
             dmax = pre_mul[c];
     }
-    FORC4 scale_mul[c] = (pre_mul[c] /= dmax) * 65535.0 / maximum;
+    FORC4 pre_mul[c] /= dmax;
     dcraw_message(dcraw, DCRAW_VERBOSE,
-    _("Scaling with darkness %d, saturation %d, and\nmultipliers"),
-    dark, sat);
+                  _("Scaling with darkness %d, saturation %d, and\nmultipliers"),
+                  black, maximum);
     FORC4 dcraw_message(dcraw, DCRAW_VERBOSE, " %f", pre_mul[c]);
     dcraw_message(dcraw, DCRAW_VERBOSE, "\n");
 
@@ -250,7 +268,7 @@ void CLASS scale_colors_INDI(int maximum, const int black,
 }
 
 void CLASS border_interpolate_INDI(const int height, const int width,
-    ushort(*image)[4], const unsigned filters, int colors, int border)
+                                   ushort(*image)[4], const unsigned filters, int colors, int border)
 {
     int row, col, y, x, f, c, sum[8];
 
@@ -262,56 +280,60 @@ void CLASS border_interpolate_INDI(const int height, const int width,
             for (y = row - 1; y != row + 2; y++)
                 for (x = col - 1; x != col + 2; x++)
                     if (y >= 0 && y < height && x >= 0 && x < width) {
-                        f = fc_INDI(filters, y, x);
-                        sum[f] += image[y*width+x][f];
-                        sum[f+4]++;
+                        f = fcol_INDI(filters, y, x);
+                        sum[f] += image[y * width + x][f];
+                        sum[f + 4]++;
                     }
-            f = fc_INDI(filters, row, col);
-            FORCC if (c != f && sum[c+4])
-                image[row*width+col][c] = sum[c] / sum[c+4];
+            f = fcol_INDI(filters, row, col);
+            FORCC if (c != f && sum[c + 4])
+                image[row * width + col][c] = sum[c] / sum[c + 4];
         }
 }
 
 void CLASS lin_interpolate_INDI(ushort(*image)[4], const unsigned filters,
-    const int width, const int height, const int colors, void *dcraw) /*UF*/
+                                const int width, const int height, const int colors, void *dcraw) /*UF*/
 {
-    int code[16][16][32], *ip, sum[4];
-    int c, i, x, y, row, col, shift, color;
+    int code[16][16][32], size = 16, *ip, sum[4];
+    int f, c, i, x, y, row, col, shift, color;
     ushort *pix;
 
     dcraw_message(dcraw, DCRAW_VERBOSE, _("Bilinear interpolation...\n")); /*UF*/
-
+#ifdef UFRAW_X_TRANS
+    if (filters == 2) size = 6;
+#endif
     border_interpolate_INDI(height, width, image, filters, colors, 1);
-    for (row = 0; row < 16; row++) {
-        for (col = 0; col < 16; col++) {
-            ip = code[row][col];
+    for (row = 0; row < size; row++) {
+        for (col = 0; col < size; col++) {
+            ip = code[row][col] + 1;
+            f = fcol_INDI(filters, row, col);
             memset(sum, 0, sizeof sum);
             for (y = -1; y <= 1; y++)
                 for (x = -1; x <= 1; x++) {
                     shift = (y == 0) + (x == 0);
-                    if (shift == 2) continue;
-                    color = fc_INDI(filters, row + y, col + x);
+                    color = fcol_INDI(filters, row + y, col + x);
+                    if (color == f) continue;
                     *ip++ = (width * y + x) * 4 + color;
                     *ip++ = shift;
                     *ip++ = color;
                     sum[color] += 1 << shift;
                 }
+            code[row][col][0] = (ip - code[row][col]) / 3;
             FORCC
-            if (c != fc_INDI(filters, row, col)) {
+            if (c != f) {
                 *ip++ = c;
                 *ip++ = 256 / sum[c];
             }
         }
     }
 #ifdef _OPENMP
-#pragma omp parallel for default(shared) private(row,col,pix,ip,sum,i)
+    #pragma omp parallel for default(shared) private(row,col,pix,ip,sum,i)
 #endif
     for (row = 1; row < height - 1; row++) {
         for (col = 1; col < width - 1; col++) {
-            pix = image[row*width+col];
-            ip = code[row & 15][col & 15];
+            pix = image[row * width + col];
+            ip = code[row % size][col % size];
             memset(sum, 0, sizeof sum);
-            for (i = 8; i--; ip += 3)
+            for (i = *ip++; i--; ip += 3)
                 sum[ip[2]] += pix[ip[0]] << ip[1];
             for (i = colors; --i; ip += 2)
                 pix[ip[0]] = sum[ip[0]] * ip[1] >> 8;
@@ -330,7 +352,7 @@ void CLASS lin_interpolate_INDI(ushort(*image)[4], const unsigned filters,
    Gradients are numbered clockwise from NW=0 to W=7.
  */
 void CLASS vng_interpolate_INDI(ushort(*image)[4], const unsigned filters,
-    const int width, const int height, const int colors, void *dcraw) /*UF*/
+                                const int width, const int height, const int colors, void *dcraw) /*UF*/
 {
     static const signed char *cp, terms[] = {
         -2, -2, +0, -1, 0, 0x01, -2, -2, +0, +0, 1, 0x01, -2, -1, -1, +0, 0, 0x01,
@@ -357,19 +379,22 @@ void CLASS vng_interpolate_INDI(ushort(*image)[4], const unsigned filters,
         +1, +0, +2, +1, 0, 0x10
     }, chood[] = { -1, -1, -1, 0, -1, +1, 0, +1, +1, +1, +1, 0, +1, -1, 0, -1 };
     ushort(*brow[4])[4], *pix;
-    int prow = 7, pcol = 1, *ip, *code[16][16], gval[8], gmin, gmax, sum[4];
+    int prow = 8, pcol = 2, *ip, *code[16][16], gval[8], gmin, gmax, sum[4];
     int row, col, x, y, x1, x2, y1, y2, t, weight, grads, color, diag;
     int g, diff, thold, num, c;
-    ushort rowtmp[4][width*4];
+    ushort rowtmp[4][width * 4];
 
     lin_interpolate_INDI(image, filters, width, height, colors, dcraw); /*UF*/
     dcraw_message(dcraw, DCRAW_VERBOSE, _("VNG interpolation...\n")); /*UF*/
 
-    if (filters == 1) prow = pcol = 15;
-    int *ipalloc = ip = (int *) calloc((prow + 1) * (pcol + 1), 1280);
+    if (filters == 1) prow = pcol = 16;
+#ifdef UFRAW_X_TRANS
+    if (filters == 2) prow = pcol =  6;
+#endif
+    int *ipalloc = ip = (int *) calloc(prow * pcol, 1280);
     merror(ip, "vng_interpolate()");
-    for (row = 0; row <= prow; row++)		/* Precalculate for VNG */
-        for (col = 0; col <= pcol; col++) {
+    for (row = 0; row < prow; row++)		/* Precalculate for VNG */
+        for (col = 0; col < pcol; col++) {
             code[row][col] = ip;
             for (cp = terms, t = 0; t < 64; t++) {
                 y1 = *cp++;
@@ -378,9 +403,9 @@ void CLASS vng_interpolate_INDI(ushort(*image)[4], const unsigned filters,
                 x2 = *cp++;
                 weight = *cp++;
                 grads = *cp++;
-                color = fc_INDI(filters, row + y1, col + x1);
-                if (fc_INDI(filters, row + y2, col + x2) != color) continue;
-                diag = (fc_INDI(filters, row, col + 1) == color && fc_INDI(filters, row + 1, col) == color) ? 2 : 1;
+                color = fcol_INDI(filters, row + y1, col + x1);
+                if (fcol_INDI(filters, row + y2, col + x2) != color) continue;
+                diag = (fcol_INDI(filters, row, col + 1) == color && fcol_INDI(filters, row + 1, col) == color) ? 2 : 1;
                 if (abs(y1 - y2) == diag && abs(x1 - x2) == diag) continue;
                 *ip++ = (y1 * width + x1) * 4 + color;
                 *ip++ = (y2 * width + x2) * 4 + color;
@@ -394,8 +419,8 @@ void CLASS vng_interpolate_INDI(ushort(*image)[4], const unsigned filters,
                 y = *cp++;
                 x = *cp++;
                 *ip++ = (y * width + x) * 4;
-                color = fc_INDI(filters, row, col);
-                if (fc_INDI(filters, row + y, col + x) != color && fc_INDI(filters, row + y * 2, col + x * 2) == color)
+                color = fcol_INDI(filters, row, col);
+                if (fcol_INDI(filters, row + y, col + x) != color && fcol_INDI(filters, row + y * 2, col + x * 2) == color)
                     *ip++ = (y * width + x) * 8 + color;
                 else
                     *ip++ = 0;
@@ -403,7 +428,7 @@ void CLASS vng_interpolate_INDI(ushort(*image)[4], const unsigned filters,
         }
     progress(PROGRESS_INTERPOLATE, -height);
 #ifdef _OPENMP
-#pragma omp parallel					\
+    #pragma omp parallel				\
     default(none)					\
     shared(image,code,prow,pcol)			\
     private(row,col,g,brow,rowtmp,pix,ip,gval,diff,gmin,gmax,thold,sum,color,num,c,t)
@@ -417,8 +442,8 @@ void CLASS vng_interpolate_INDI(ushort(*image)[4], const unsigned filters,
             for (g = 0; g < 4; g++)
                 brow[g] = &rowtmp[(row + g - 2) % 4];
             for (col = 2; col < width - 2; col++) {
-                pix = image[row*width+col];
-                ip = code[row & prow][col & pcol];
+                pix = image[row * width + col];
+                ip = code[row % prow][col % pcol];
                 memset(gval, 0, sizeof gval);
                 while ((g = ip[0]) != INT_MAX) { /* Calculate gradients */
                     diff = ABS(pix[g] - pix[ip[1]]) << ip[2];
@@ -441,7 +466,7 @@ void CLASS vng_interpolate_INDI(ushort(*image)[4], const unsigned filters,
                 }
                 thold = gmin + (gmax >> 1);
                 memset(sum, 0, sizeof sum);
-                color = fc_INDI(filters, row, col);
+                color = fcol_INDI(filters, row, col);
                 for (num = g = 0; g < 8; g++, ip += 2) { /* Average the neighbors */
                     if (gval[g] <= thold) {
                         FORCC
@@ -459,12 +484,13 @@ void CLASS vng_interpolate_INDI(ushort(*image)[4], const unsigned filters,
                     brow[2][col][c] = CLIP(t);
                 }
             }
-            if (row > start_row + 1)		/* Write buffer to image */
-                memcpy(image[(row-2)*width+2], brow[0] + 2, (width - 4)*sizeof * image);
-        }
-        if (row == height - 2) {
-            memcpy(image[(row-2)*width+2], brow[0] + 2, (width - 4)*sizeof * image);
-            memcpy(image[(row-1)*width+2], brow[1] + 2, (width - 4)*sizeof * image);
+            /* Write buffer to image */
+            if ((row > start_row + 1) || (row == height - 2))
+                memcpy(image[(row - 2)*width + 2], brow[0] + 2, (width - 4)*sizeof * image);
+            if (row == height - 2) {
+                memcpy(image[(row - 1)*width + 2], brow[1] + 2, (width - 4)*sizeof * image);
+                break;
+            }
         }
     }
     free(ipalloc);
@@ -474,7 +500,8 @@ void CLASS vng_interpolate_INDI(ushort(*image)[4], const unsigned filters,
    Patterned Pixel Grouping Interpolation by Alain Desbiolles
 */
 void CLASS ppg_interpolate_INDI(ushort(*image)[4], const unsigned filters,
-    const int width, const int height, const int colors, void *dcraw)
+                                const int width, const int height,
+                                const int colors, void *dcraw)
 {
     int dir[5] = { 1, width, -1, -width, 1 };
     int row, col, diff[2], guess[2], c, d, i;
@@ -484,27 +511,27 @@ void CLASS ppg_interpolate_INDI(ushort(*image)[4], const unsigned filters,
     dcraw_message(dcraw, DCRAW_VERBOSE, _("PPG interpolation...\n")); /*UF*/
 
 #ifdef _OPENMP
-#pragma omp parallel					\
-  default(none)						\
-  shared(image,dir)					\
-  private(row,col,i,d,c,pix,diff,guess)
+    #pragma omp parallel				\
+    default(none)					\
+    shared(image,dir)					\
+    private(row,col,i,d,c,pix,diff,guess)
 #endif
     {
         /*  Fill in the green layer with gradients and pattern recognition: */
 #ifdef _OPENMP
-#pragma omp for
+        #pragma omp for
 #endif
         for (row = 3; row < height - 3; row++) {
             for (col = 3 + (FC(row, 3) & 1), c = FC(row, col); col < width - 3; col += 2) {
                 pix = image + row * width + col;
                 for (i = 0; (d = dir[i]) > 0; i++) {
                     guess[i] = (pix[-d][1] + pix[0][c] + pix[d][1]) * 2
-                               - pix[-2*d][c] - pix[2*d][c];
-                    diff[i] = (ABS(pix[-2*d][c] - pix[ 0][c]) +
-                               ABS(pix[ 2*d][c] - pix[ 0][c]) +
+                               - pix[-2 * d][c] - pix[2 * d][c];
+                    diff[i] = (ABS(pix[-2 * d][c] - pix[ 0][c]) +
+                               ABS(pix[ 2 * d][c] - pix[ 0][c]) +
                                ABS(pix[  -d][1] - pix[ d][1])) * 3 +
-                              (ABS(pix[ 3*d][1] - pix[ d][1]) +
-                               ABS(pix[-3*d][1] - pix[-d][1])) * 2;
+                              (ABS(pix[ 3 * d][1] - pix[ d][1]) +
+                               ABS(pix[-3 * d][1] - pix[-d][1])) * 2;
                 }
                 d = dir[i = diff[0] > diff[1]];
                 pix[0][1] = ULIM(guess[i] >> 2, pix[d][1], pix[-d][1]);
@@ -512,7 +539,7 @@ void CLASS ppg_interpolate_INDI(ushort(*image)[4], const unsigned filters,
         }
         /*  Calculate red and blue for each green pixel: */
 #ifdef _OPENMP
-#pragma omp for
+        #pragma omp for
 #endif
         for (row = 1; row < height - 1; row++) {
             for (col = 1 + (FC(row, 2) & 1), c = FC(row, col + 1); col < width - 1; col += 2) {
@@ -524,12 +551,12 @@ void CLASS ppg_interpolate_INDI(ushort(*image)[4], const unsigned filters,
         }
         /*  Calculate blue for red pixels and vice versa: */
 #ifdef _OPENMP
-#pragma omp for
+        #pragma omp for
 #endif
         for (row = 1; row < height - 1; row++) {
             for (col = 1 + (FC(row, 1) & 1), c = 2 - FC(row, col); col < width - 1; col += 2) {
                 pix = image + row * width + col;
-                for (i = 0; (d = dir[i] + dir[i+1]) > 0; i++) {
+                for (i = 0; (d = dir[i] + dir[i + 1]) > 0; i++) {
                     diff[i] = ABS(pix[-d][c] - pix[d][c]) +
                               ABS(pix[-d][1] - pix[0][1]) +
                               ABS(pix[ d][1] - pix[0][1]);
@@ -552,8 +579,9 @@ void CLASS ppg_interpolate_INDI(ushort(*image)[4], const unsigned filters,
 #define TS 256 /* Tile Size */
 
 void CLASS ahd_interpolate_INDI(ushort(*image)[4], const unsigned filters,
-    const int width, const int height, const int colors,
-    const float rgb_cam[3][4], void *dcraw)
+                                const int width, const int height,
+                                const int colors, const float rgb_cam[3][4],
+                                void *dcraw)
 {
     int i, j, k, top, left, row, col, tr, tc, c, d, val, hm[2];
     ushort(*pix)[4], (*rix)[3];
@@ -567,20 +595,20 @@ void CLASS ahd_interpolate_INDI(ushort(*image)[4], const unsigned filters,
     dcraw_message(dcraw, DCRAW_VERBOSE, _("AHD interpolation...\n")); /*UF*/
 
 #ifdef _OPENMP
-#pragma omp parallel					\
-  default(shared)					\
-  private(top, left, row, col, pix, rix, lix, c, xyz, val, d, tc, tr, i, j, k, ldiff, abdiff, leps, abeps, hm, buffer, rgb, lab, homo, r)
+    #pragma omp parallel				\
+    default(shared)					\
+    private(top, left, row, col, pix, rix, lix, c, xyz, val, d, tc, tr, i, j, k, ldiff, abdiff, leps, abeps, hm, buffer, rgb, lab, homo, r)
 #endif
     {
 #ifdef _OPENMP
-#pragma omp for schedule(static) nowait
+        #pragma omp for schedule(static) nowait
 #endif
         for (i = 0; i < 0x10000; i++) {
             r = i / 65535.0;
             cbrt[i] = r > 0.008856 ? pow(r, 1 / 3.0) : 7.787 * r + 16 / 116.0;
         }
 #ifdef _OPENMP
-#pragma omp for
+        #pragma omp for
 #endif
         for (i = 0; i < 3; i++)
             for (j = 0; j < colors; j++)
@@ -596,7 +624,7 @@ void CLASS ahd_interpolate_INDI(ushort(*image)[4], const unsigned filters,
 
         progress(PROGRESS_INTERPOLATE, -height);
 #ifdef _OPENMP
-#pragma omp for
+        #pragma omp for
 #endif
         for (top = 2; top < height - 5; top += TS - 6) {
             progress(PROGRESS_INTERPOLATE, TS - 6);
@@ -609,10 +637,10 @@ void CLASS ahd_interpolate_INDI(ushort(*image)[4], const unsigned filters,
                         pix = image + row * width + col;
                         val = ((pix[-1][1] + pix[0][c] + pix[1][1]) * 2
                                - pix[-2][c] - pix[2][c]) >> 2;
-                        rgb[0][row-top][col-left][1] = ULIM(val, pix[-1][1], pix[1][1]);
+                        rgb[0][row - top][col - left][1] = ULIM(val, pix[-1][1], pix[1][1]);
                         val = ((pix[-width][1] + pix[0][c] + pix[width][1]) * 2
-                               - pix[-2*width][c] - pix[2*width][c]) >> 2;
-                        rgb[1][row-top][col-left][1] = ULIM(val, pix[-width][1], pix[width][1]);
+                               - pix[-2 * width][c] - pix[2 * width][c]) >> 2;
+                        rgb[1][row - top][col - left][1] = ULIM(val, pix[-width][1], pix[width][1]);
                     }
                 }
                 /*  Interpolate red and blue, and convert to CIELab: */
@@ -620,20 +648,20 @@ void CLASS ahd_interpolate_INDI(ushort(*image)[4], const unsigned filters,
                     for (row = top + 1; row < top + TS - 1 && row < height - 3; row++)
                         for (col = left + 1; col < left + TS - 1 && col < width - 3; col++) {
                             pix = image + row * width + col;
-                            rix = &rgb[d][row-top][col-left];
-                            lix = &lab[d][row-top][col-left];
+                            rix = &rgb[d][row - top][col - left];
+                            lix = &lab[d][row - top][col - left];
                             if ((c = 2 - FC(row, col)) == 1) {
                                 c = FC(row + 1, col);
-                                val = pix[0][1] + ((pix[-1][2-c] + pix[1][2-c]
+                                val = pix[0][1] + ((pix[-1][2 - c] + pix[1][2 - c]
                                                     - rix[-1][1] - rix[1][1]) >> 1);
-                                rix[0][2-c] = CLIP(val);
+                                rix[0][2 - c] = CLIP(val);
                                 val = pix[0][1] + ((pix[-width][c] + pix[width][c]
                                                     - rix[-TS][1] - rix[TS][1]) >> 1);
                             } else
-                                val = rix[0][1] + ((pix[-width-1][c] + pix[-width+1][c]
-                                                    + pix[+width-1][c] + pix[+width+1][c]
-                                                    - rix[-TS-1][1] - rix[-TS+1][1]
-                                                    - rix[+TS-1][1] - rix[+TS+1][1] + 1) >> 2);
+                                val = rix[0][1] + ((pix[-width - 1][c] + pix[-width + 1][c]
+                                                    + pix[+width - 1][c] + pix[+width + 1][c]
+                                                    - rix[-TS - 1][1] - rix[-TS + 1][1]
+                                                    - rix[+TS - 1][1] - rix[+TS + 1][1] + 1) >> 2);
                             rix[0][c] = CLIP(val);
                             c = FC(row, col);
                             rix[0][c] = pix[0][c];
@@ -684,9 +712,9 @@ void CLASS ahd_interpolate_INDI(ushort(*image)[4], const unsigned filters,
                                 for (j = tc - 1; j <= tc + 1; j++)
                                     hm[d] += homo[d][i][j];
                         if (hm[0] != hm[1])
-                            FORC3 image[row*width+col][c] = rgb[hm[1] > hm[0]][tr][tc][c];
+                            FORC3 image[row * width + col][c] = rgb[hm[1] > hm[0]][tr][tc][c];
                         else
-                            FORC3 image[row*width+col][c] =
+                            FORC3 image[row * width + col][c] =
                                 (rgb[0][tr][tc][c] + rgb[1][tr][tc][c]) >> 1;
                     }
                 }
@@ -741,7 +769,7 @@ static inline int median9(int *p)
 
 // Just making this function inline speeds up ahd_interpolate_INDI() up to 15%
 static inline ushort eahd_median(int row, int col, int color,
-    ushort(*image)[4], const int width)
+                                 ushort(*image)[4], const int width)
 {
     //declare the pixel array
     int pArray[9];
@@ -754,18 +782,18 @@ static inline ushort eahd_median(int row, int col, int color,
     //  results = (median(G-R)+median(G-B)+R+B)/2
 
     //no checks are done here to speed up the inlining
-    pArray[0] = image[width*(row)+col+1][color] - image[width*(row)+col+1][1];
-    pArray[1] = image[width*(row-1)+col+1][color] - image[width*(row-1)+col+1][1];
-    pArray[2] = image[width*(row-1)+col  ][color] - image[width*(row-1)+col  ][1];
-    pArray[3] = image[width*(row-1)+col-1][color] - image[width*(row-1)+col-1][1];
-    pArray[4] = image[width*(row)+col-1][color] - image[width*(row)+col-1][1];
-    pArray[5] = image[width*(row+1)+col-1][color] - image[width*(row+1)+col-1][1];
-    pArray[6] = image[width*(row+1)+col  ][color] - image[width*(row+1)+col  ][1];
-    pArray[7] = image[width*(row+1)+col+1][color] - image[width*(row+1)+col+1][1];
-    pArray[8] = image[width*(row)+col  ][color] - image[width*(row)+col  ][1];
+    pArray[0] = image[width * (row) + col + 1][color] - image[width * (row) + col + 1][1];
+    pArray[1] = image[width * (row - 1) + col + 1][color] - image[width * (row - 1) + col + 1][1];
+    pArray[2] = image[width * (row - 1) + col  ][color] - image[width * (row - 1) + col  ][1];
+    pArray[3] = image[width * (row - 1) + col - 1][color] - image[width * (row - 1) + col - 1][1];
+    pArray[4] = image[width * (row) + col - 1][color] - image[width * (row) + col - 1][1];
+    pArray[5] = image[width * (row + 1) + col - 1][color] - image[width * (row + 1) + col - 1][1];
+    pArray[6] = image[width * (row + 1) + col  ][color] - image[width * (row + 1) + col  ][1];
+    pArray[7] = image[width * (row + 1) + col + 1][color] - image[width * (row + 1) + col + 1][1];
+    pArray[8] = image[width * (row) + col  ][color] - image[width * (row) + col  ][1];
 
     median9(pArray);
-    result = pArray[4] + image[width*(row)+col  ][1];
+    result = pArray[4] + image[width * (row) + col  ][1];
     return DTOP(result);
 
 }
@@ -773,7 +801,7 @@ static inline ushort eahd_median(int row, int col, int color,
 // Add the color smoothing from Kimmel as suggested in the AHD paper
 // Algorithm updated by Michael Goertz
 void CLASS color_smooth(ushort(*image)[4], const int width, const int height,
-    const int passes)
+                        const int passes)
 {
     int row, col;
     int row_start = 2;
@@ -788,13 +816,13 @@ void CLASS color_smooth(ushort(*image)[4], const int width, const int height,
     for (count = 0; count < passes; count++) {
         //perform 3 iterations - seems to be a commonly settled upon number of iterations
 #ifdef _OPENMP
-#pragma omp parallel for default(shared) private(row,col,mpix)
+        #pragma omp parallel for default(shared) private(row,col,mpix)
 #endif
         for (row = row_start; row < row_stop; row++) {
             for (col = col_start; col < col_stop; col++) {
                 //calculate the median only over the red and blue
                 //calculating over green seems to offer very little additional quality
-                mpix = image[row*width+col];
+                mpix = image[row * width + col];
                 mpix[0] = eahd_median(row, col, 0, image, width);
                 mpix[2] = eahd_median(row, col, 2, image, width);
             }
@@ -803,13 +831,12 @@ void CLASS color_smooth(ushort(*image)[4], const int width, const int height,
 }
 
 void CLASS fuji_rotate_INDI(ushort(**image_p)[4], int *height_p,
-    int *width_p, int *fuji_width_p, const int colors, const double step,
-    void *dcraw)
+                            int *width_p, int *fuji_width_p, const int colors,
+                            const double step, void *dcraw)
 {
     int height = *height_p, width = *width_p, fuji_width = *fuji_width_p; /*UF*/
     ushort(*image)[4] = *image_p;  /*UF*/
     int i, row, col;
-//  double step;
     float r, c, fr, fc;
     int ur, uc;
     ushort wide, high, (*img)[4], (*pix)[4];
@@ -817,14 +844,13 @@ void CLASS fuji_rotate_INDI(ushort(**image_p)[4], int *height_p,
     if (!fuji_width) return;
     dcraw_message(dcraw, DCRAW_VERBOSE, _("Rotating image 45 degrees...\n"));
     fuji_width = (fuji_width - 1/* + shrink*/)/* >> shrink*/;
-//  step = sqrt(0.5);
     wide = fuji_width / step;
     high = (height - fuji_width) / step;
     img = (ushort(*)[4]) calloc(wide * high, sizeof * img);
     merror(img, "fuji_rotate()");
 
 #ifdef _OPENMP
-#pragma omp parallel for default(shared) private(row,col,ur,uc,r,c,fr,fc,pix,i)
+    #pragma omp parallel for default(shared) private(row,col,ur,uc,r,c,fr,fc,pix,i)
 #endif
     for (row = 0; row < high; row++) {
         for (col = 0; col < wide; col++) {
@@ -835,9 +861,9 @@ void CLASS fuji_rotate_INDI(ushort(**image_p)[4], int *height_p,
             fc = c - uc;
             pix = image + ur * width + uc;
             for (i = 0; i < colors; i++)
-                img[row*wide+col][i] =
+                img[row * wide + col][i] =
                     (pix[    0][i] * (1 - fc) + pix[      1][i] * fc) * (1 - fr) +
-                    (pix[width][i] * (1 - fc) + pix[width+1][i] * fc) * fr;
+                    (pix[width][i] * (1 - fc) + pix[width + 1][i] * fc) * fr;
         }
     }
     free(image);
@@ -852,7 +878,7 @@ void CLASS fuji_rotate_INDI(ushort(**image_p)[4], int *height_p,
 }
 
 void CLASS flip_image_INDI(ushort(*image)[4], int *height_p, int *width_p,
-    /*const*/ int flip) /*UF*/
+                           /*const*/ int flip) /*UF*/
 {
     unsigned *flag;
     int size, base, dest, next, row, col;
