@@ -2,7 +2,7 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * ufraw-gimp.c - The GIMP plug-in.
- * Copyright 2004-2011 by Udi Fuchs
+ * Copyright 2004-2013 by Udi Fuchs
  *
  * based on the GIMP plug-in by Pawel T. Jochym jochym at ifj edu pl,
  *
@@ -25,14 +25,25 @@
 /* Fix some compatibility issues between CinePaint and GIMP */
 typedef GimpRunModeType GimpRunMode;
 #define PLUGIN_MODE 2
+#define DEPTH_TO_BASETYPE(depth) (depth == 3 ? RGB : U16_RGB)
+#define DEPTH_TO_IMAGETYPE(depth) (depth == 3 ? RGB_IMAGE : U16_RGB_IMAGE)
+#else /* GIMP */
+#if HAVE_GIMP_2_9
+#include <gegl.h>
+#define PLUGIN_MODE 2
 #else
+#define PLUGIN_MODE 1
+#endif
+#include <libgimpbase/gimpbase.h>
 #include <libgimp/gimp.h>
 #include <libgimp/gimpui.h>
 #define GIMP_CONST const
 /* Missing and irrelevant definitions in GIMP */
 #define U16_RGB 0
 #define U16_RGB_IMAGE 0
-#define PLUGIN_MODE 1
+#define DEPTH_TO_BASETYPE(depth) GIMP_RGB
+#define DEPTH_TO_IMAGETYPE(depth) GIMP_RGB_IMAGE
+#define DEPTH_TO_PRECISION(depth) (depth == 3 ? GIMP_PRECISION_U8 : GIMP_PRECISION_U16)
 #endif
 #include <glib/gi18n.h>
 #include <string.h>
@@ -82,7 +93,7 @@ void query()
                            "Udi Fuchs",
                            "Copyright 2003 by Dave Coffin\n"
                            "Copyright 2004 by Pawel Jochym\n"
-                           "Copyright 2004-2011 by Udi Fuchs",
+                           "Copyright 2004-2013 by Udi Fuchs",
                            "ufraw-" VERSION,
 #ifndef UFRAW_CINEPAINT
                            "raw image",
@@ -103,7 +114,7 @@ void query()
                            "Loads thumbnails from digital camera raw files.",
                            "Loads thumbnails from digital camera raw files.",
                            "Udi Fuchs",
-                           "Copyright 2004-2011 by Udi Fuchs",
+                           "Copyright 2004-2013 by Udi Fuchs",
                            "ufraw-" VERSION,
                            NULL,
                            NULL,
@@ -146,11 +157,16 @@ void run(GIMP_CONST gchar *name,
     conf_data rc;
     int status;
 
+#if !GLIB_CHECK_VERSION(2,31,0)
     g_thread_init(NULL);
+#endif
     gdk_threads_init();
     gdk_threads_enter();
     ufraw_binary = g_path_get_basename(gimp_get_progname());
     uf_init_locale(gimp_get_progname());
+#if HAVE_GIMP_2_9
+    gegl_init(NULL, NULL);
+#endif
 
     *nreturn_vals = 1;
     *return_vals = values;
@@ -236,7 +252,7 @@ void run(GIMP_CONST gchar *name,
 
     ufraw_config(uf, &rc, NULL, NULL);
     sendToGimpMode = (uf->conf->createID == send_id);
-#ifndef UFRAW_CINEPAINT
+#if !defined(UFRAW_CINEPAINT) && !HAVE_GIMP_2_9
     if (loadThumbnail) {
         uf->conf->size = size;
         uf->conf->embeddedImage = TRUE;
@@ -272,6 +288,7 @@ void run(GIMP_CONST gchar *name,
         if (sendToGimpMode) gimp_progress_update(0.3);
         ufraw_save_gimp_image(uf, NULL);
         if (sendToGimpMode) gimp_progress_update(1.0);
+        ufraw_close_darkframe(uf->conf);
         ufraw_close(uf);
         /* To make sure we don't delete the raw file by mistake we check
          * that the file is really an ID file. */
@@ -311,18 +328,29 @@ int gimp_row_writer(ufraw_data *uf, void *volatile out, void *pixbuf,
     (void)grayscale;
     (void)bitDepth;
 
+#if HAVE_GIMP_2_9
+    gegl_buffer_set(out, GEGL_RECTANGLE(0, row, width, height),
+                    0, NULL, pixbuf,
+                    GEGL_AUTO_ROWSTRIDE);
+#else
     gimp_pixel_rgn_set_rect(out, pixbuf, 0, row, width, height);
+#endif
 
     return UFRAW_SUCCESS;
 }
 
 long ufraw_save_gimp_image(ufraw_data *uf, GtkWidget *widget)
 {
+#if HAVE_GIMP_2_9
+    GeglBuffer *buffer;
+#else
     GimpDrawable *drawable;
     GimpPixelRgn pixel_region;
+    int tile_height, row, nrows;
+#endif
     gint32 layer;
     UFRectangle Crop;
-    int depth, tile_height, row, nrows;
+    int depth;
     (void)widget;
 
     uf->gimpImage = -1;
@@ -339,7 +367,7 @@ long ufraw_save_gimp_image(ufraw_data *uf, GtkWidget *widget)
         if (ufraw_convert_image(uf) != UFRAW_SUCCESS)
             return UFRAW_ERROR;
         ufraw_get_scaled_crop(uf, &Crop);
-#ifdef UFRAW_CINEPAINT
+#if defined(UFRAW_CINEPAINT) || HAVE_GIMP_2_9
         if (uf->conf->profile[out_profile]
                 [uf->conf->profileIndex[out_profile]].BitDepth == 16)
             depth = 6;
@@ -349,8 +377,15 @@ long ufraw_save_gimp_image(ufraw_data *uf, GtkWidget *widget)
         depth = 3;
 #endif
     }
+#if HAVE_GIMP_2_9
+    uf->gimpImage =
+        gimp_image_new_with_precision(Crop.width, Crop.height,
+                                      DEPTH_TO_BASETYPE(depth),
+                                      DEPTH_TO_PRECISION(depth));
+#else
     uf->gimpImage = gimp_image_new(Crop.width, Crop.height,
-                                   depth == 3 ? GIMP_RGB : U16_RGB);
+                                   DEPTH_TO_BASETYPE(depth));
+#endif
     if (uf->gimpImage == -1) {
         ufraw_message(UFRAW_ERROR, _("Can't allocate new image."));
         return UFRAW_ERROR;
@@ -359,28 +394,56 @@ long ufraw_save_gimp_image(ufraw_data *uf, GtkWidget *widget)
 
     /* Create the "background" layer to hold the image... */
     layer = gimp_layer_new(uf->gimpImage, _("Background"), Crop.width,
-                           Crop.height, depth == 3 ? GIMP_RGB_IMAGE : U16_RGB_IMAGE,
+                           Crop.height, DEPTH_TO_IMAGETYPE(depth),
                            100.0, GIMP_NORMAL_MODE);
+#ifdef UFRAW_CINEPAINT
     gimp_image_add_layer(uf->gimpImage, layer, 0);
+#else
+#if defined(GIMP_CHECK_VERSION) && GIMP_CHECK_VERSION(2,7,3)
+    gimp_image_insert_layer(uf->gimpImage, layer, 0, 0);
+#else
+    gimp_image_add_layer(uf->gimpImage, layer, 0);
+#endif
+#endif
 
     /* Get the drawable and set the pixel region for our load... */
+#if HAVE_GIMP_2_9
+    buffer = gimp_drawable_get_buffer(layer);
+#else
     drawable = gimp_drawable_get(layer);
     gimp_pixel_rgn_init(&pixel_region, drawable, 0, 0, drawable->width,
                         drawable->height, TRUE, FALSE);
     tile_height = gimp_tile_height();
+#endif
 
     if (uf->conf->embeddedImage) {
+#if HAVE_GIMP_2_9
+        gegl_buffer_set(buffer,
+                        GEGL_RECTANGLE(0, 0, Crop.width, Crop.height),
+                        0, NULL, uf->thumb.buffer,
+                        GEGL_AUTO_ROWSTRIDE);
+#else
         for (row = 0; row < Crop.height; row += tile_height) {
             nrows = MIN(Crop.height - row, tile_height);
             gimp_pixel_rgn_set_rect(&pixel_region,
                                     uf->thumb.buffer + 3 * row * Crop.width, 0, row, Crop.width, nrows);
         }
+#endif
     } else {
+#if HAVE_GIMP_2_9
+        ufraw_write_image_data(uf, buffer, &Crop, depth == 3 ? 8 : 16, 0,
+                               gimp_row_writer);
+#else
         ufraw_write_image_data(uf, &pixel_region, &Crop, depth == 3 ? 8 : 16, 0,
                                gimp_row_writer);
+#endif
     }
+#if HAVE_GIMP_2_9
+    gegl_buffer_flush(buffer);
+#else
     gimp_drawable_flush(drawable);
     gimp_drawable_detach(drawable);
+#endif
 
     if (uf->conf->embeddedImage) return UFRAW_SUCCESS;
 
