@@ -2,7 +2,7 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * ufraw.c - The standalone interface to UFRaw.
- * Copyright 2004-2011 by Udi Fuchs
+ * Copyright 2004-2013 by Udi Fuchs
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,7 +28,9 @@ int main(int argc, char **argv)
     int optInd;
     int plugin = 0;
 
+#if !GLIB_CHECK_VERSION(2,31,0)
     g_thread_init(NULL);
+#endif
     gdk_threads_init();
     gdk_threads_enter();
     char *argFile = uf_win32_locale_to_utf8(argv[0]);
@@ -66,11 +68,12 @@ int main(int argc, char **argv)
     }
     if (cmd.silent) {
         ufraw_message(UFRAW_ERROR,
-                      _("--silent option is valid only in batch mode"));
+                      _("The --silent option is only valid with 'ufraw-batch'"));
         optInd = -1;
     }
     if (cmd.embeddedImage) {
-        ufraw_message(UFRAW_ERROR, _("Extracting embedded image is not supported in interactive mode"));
+        ufraw_message(UFRAW_ERROR,
+                      _("The --embedded-image option is only valid with 'ufraw-batch'"));
         optInd = -1;
     }
     if (optInd < 0) {
@@ -119,6 +122,7 @@ int main(int argc, char **argv)
         }
         status = ufraw_config(uf, &rc, &conf, &cmd);
         if (status == UFRAW_ERROR) {
+            ufraw_close_darkframe(uf->conf);
             ufraw_close(uf);
             g_free(uf);
             gdk_threads_leave();
