@@ -3,7 +3,7 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * ufraw.h - Common definitions for UFRaw.
- * Copyright 2004-2011 by Udi Fuchs
+ * Copyright 2004-2013 by Udi Fuchs
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -85,7 +85,7 @@ extern "C" {
     UFObject *ufraw_image_new();
 #ifdef HAVE_LENSFUN
     UFObject *ufraw_lensfun_new();
-    void ufraw_lensfun_init(UFObject *lensfun);
+    void ufraw_lensfun_init(UFObject *lensfun, UFBoolean reset);
     struct lfDatabase *ufraw_lensfun_db(); /* mount/camera/lens database */
     const struct lfCamera *ufraw_lensfun_camera(const UFObject *lensfun);
     void ufraw_lensfun_set_camera(UFObject *lensfun, const struct lfCamera *camera);
@@ -220,7 +220,7 @@ typedef struct {
  * CONF|ID: curve/profile are added to the list from RC.
  * CONF: inputFilename, outputFilename are ignored.
  * outputPath can only be specified in CMD or guessed in interactive mode.
- * ID: createID==only_id is switched to also_id in case of ufraw-batch.
+ * ID: createID==only_id is switched to no_id in case of ufraw-batch.
  * ID: chanMul[] override wb, green, temperature.
  */
 typedef struct {
@@ -252,6 +252,7 @@ typedef struct {
     char darkframeFile[max_path];
     struct ufraw_struct *darkframe;
     int CropX1, CropY1, CropX2, CropY2;
+    double aspectRatio;
     int orientation;
     double rotationAngle;
     int lightnessAdjustmentCount;
@@ -379,6 +380,7 @@ extern "C" {
                                       gboolean bufferok);
     ufraw_image_data *ufraw_convert_image_area(ufraw_data *uf, unsigned saidx,
             UFRawPhase phase);
+    void ufraw_close_darkframe(conf_data *uf);
     void ufraw_close(ufraw_data *uf);
     void ufraw_flip_orientation(ufraw_data *uf, int flip);
     void ufraw_flip_image(ufraw_data *uf, int flip);
