@@ -2,7 +2,7 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * ufraw_routines.c - general routines
- * Copyright 2004-2011 by Udi Fuchs
+ * Copyright 2004-2013 by Udi Fuchs
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -165,12 +165,14 @@ char *uf_markup_buf(char *buffer, const char *format, ...)
     }
 }
 
-const char raw_ext[] = "3fr,arw,bay,bmq,cine,cr2,crw,cs1,dc2,dcr,dng,erf,fff,"
-                       "hdr,ia,jpg,k25,kc2,kdc,mdc,mef,mos,mrw,nef,nrw,orf,pef,pxn,qtk,raf,"
-                       "raw,rdc,rw2,rwl,sr2,srf,srw,sti,tif,ufraw,x3f";
+const char raw_ext[] = "3fr,ari,arw,cap,cine,cr2,crw,cs1,dc2,dcr,dng,erf,fff,"
+                       "hdr,ia,iiq,jpeg,jpg,k25,kc2,kdc,mdc,mef,mos,mrw,nef,"
+                       "nrw,orf,pef,pxn,qtk,r3d,raf,raw,rdc,rw2,rwl,sr2,srf,"
+                       "srw,sti,tif,tiff,ufraw,x3f";
 
 const char *file_type[] = { ".ppm", ".ppm", ".tif", ".tif", ".jpg",
-                            ".png", ".png", ".embedded.jpg", ".embedded.png", ".fits"
+                            ".png", ".png", ".embedded.jpg", ".embedded.png",
+                            ".fits"
                           };
 
 /* Set locale of LC_NUMERIC to "C" to make sure that printf behaves correctly.*/
@@ -231,7 +233,7 @@ double profile_default_gamma(profile_data *p)
 
 /* Convert between Temperature and RGB.
  * Base on information from http://www.brucelindbloom.com/
- * The fit for D-illuminant between 4000K and 15000K are from CIE
+ * The fit for D-illuminant between 4000K and 23000K are from CIE
  * The generalization to 2000K < T < 4000K and the blackbody fits
  * are my own and should be taken with a grain of salt.
  */
@@ -278,7 +280,7 @@ void RGB_to_Temperature(double RGB[3], double *T, double *Green)
 {
     double Tmax, Tmin, testRGB[3];
     Tmin = 2000;
-    Tmax = 15000;
+    Tmax = 23000;
     for (*T = (Tmax + Tmin) / 2; Tmax - Tmin > 0.1; *T = (Tmax + Tmin) / 2) {
         Temperature_to_RGB(*T, testRGB);
         if (testRGB[2] / testRGB[0] > RGB[2] / RGB[0])
@@ -342,7 +344,7 @@ static void curve_parse_text(GMarkupParseContext *context, const gchar *text,
     char temp[max_path];
     error = error;
     for (; len > 0 && g_ascii_isspace(*text); len--, text++);
-    for (; len > 0 && g_ascii_isspace(text[len-1]); len--);
+    for (; len > 0 && g_ascii_isspace(text[len - 1]); len--);
     if (len == 0) return;
     if (len > max_path - 1) len = max_path - 1;
     strncpy(temp, text, len);
@@ -407,8 +409,14 @@ int curve_load(CurveData *cp, char *filename)
         }
         locale = uf_set_locale_C();
         context = g_markup_parse_context_new(&parser, 0, cp, NULL);
-        line[max_path-1] = '\0';
-        char *dummy = fgets(line, max_path - 1, in);
+        line[max_path - 1] = '\0';
+        if (fgets(line, max_path - 1, in) == NULL && !feof(in)) {
+            ufraw_message(UFRAW_ERROR, _("Error reading from file '%s'."),
+                          filename);
+            uf_reset_locale(locale);
+            fclose(in);
+            return UFRAW_ERROR;
+        }
         while (!feof(in)) {
             if (!g_markup_parse_context_parse(context, line,
                                               strlen(line), &err)) {
@@ -420,7 +428,13 @@ int curve_load(CurveData *cp, char *filename)
                 g_error_free(err);
                 return UFRAW_ERROR;
             }
-            dummy = fgets(line, max_path, in);
+            if (fgets(line, max_path, in) == NULL && !feof(in)) {
+                ufraw_message(UFRAW_ERROR, _("Error reading from file '%s'."),
+                              filename);
+                uf_reset_locale(locale);
+                fclose(in);
+                return UFRAW_ERROR;
+            }
         }
         g_markup_parse_context_end_parse(context, NULL);
         g_markup_parse_context_free(context);
