@@ -2,7 +2,7 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * ufraw_ufraw.c - program interface to all the components
- * Copyright 2004-2011 by Udi Fuchs
+ * Copyright 2004-2013 by Udi Fuchs
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,11 +38,12 @@ void (*ufraw_progress)(int what, int ticks) = NULL;
 static void ufraw_convert_image_vignetting(ufraw_data *uf,
         ufraw_image_data *img, UFRectangle *area);
 static void ufraw_convert_image_tca(ufraw_data *uf, ufraw_image_data *img,
-                                    ufraw_image_data *outimg, UFRectangle *area);
+                                    ufraw_image_data *outimg,
+                                    UFRectangle *area);
 void ufraw_prepare_tca(ufraw_data *uf);
 #endif
 static void ufraw_image_format(int *colors, int *bytes, ufraw_image_data *img,
-                               const char *formats, const char *caller);
+        const char *formats, const char *caller);
 static void ufraw_convert_image_raw(ufraw_data *uf, UFRawPhase phase);
 static void ufraw_convert_image_first(ufraw_data *uf, UFRawPhase phase);
 static void ufraw_convert_image_transform(ufraw_data *uf, ufraw_image_data *img,
@@ -52,7 +53,8 @@ static void ufraw_convert_prepare_first_buffer(ufraw_data *uf,
 static void ufraw_convert_prepare_transform_buffer(ufraw_data *uf,
         ufraw_image_data *img, int width, int height);
 static void ufraw_convert_reverse_wb(ufraw_data *uf, UFRawPhase phase);
-static void ufraw_convert_import_buffer(ufraw_data *uf, UFRawPhase phase, dcraw_image_data *dcimg);
+static void ufraw_convert_import_buffer(ufraw_data *uf, UFRawPhase phase,
+                                        dcraw_image_data *dcimg);
 
 static int make_temporary(char *basefilename, char **tmpfilename)
 {
@@ -281,7 +283,7 @@ int ufraw_load_darkframe(ufraw_data *uf)
         if (strcmp(uf->conf->darkframeFile, uf->conf->darkframe->filename) == 0)
             return UFRAW_SUCCESS;
         // Otherwise we need to close the previous darkframe
-        ufraw_close(uf->conf->darkframe);
+        ufraw_close_darkframe(uf->conf);
     }
     ufraw_data *dark = uf->conf->darkframe =
                            ufraw_open(uf->conf->darkframeFile);
@@ -293,6 +295,8 @@ int ufraw_load_darkframe(ufraw_data *uf)
     }
     dark->conf = g_new(conf_data, 1);
     conf_init(dark->conf);
+    /* initialize ufobject member */
+    dark->conf->ufobject = ufraw_image_new();
     /* disable all auto settings on darkframe */
     dark->conf->autoExposure = disabled_state;
     dark->conf->autoBlack = disabled_state;
@@ -359,6 +363,35 @@ void ufraw_get_image_dimensions(ufraw_data *uf)
     if (uf->conf->CropY1 < 0) uf->conf->CropY1 = 0;
     if (uf->conf->CropX2 < 0) uf->conf->CropX2 = uf->rotatedWidth;
     if (uf->conf->CropY2 < 0) uf->conf->CropY2 = uf->rotatedHeight;
+
+    if (uf->conf->aspectRatio <= 0) {
+        if (uf->conf->autoCrop)
+            /* preserve the initial aspect ratio - this should be consistent
+               with ufraw_convert_prepare_transform */
+            uf->conf->aspectRatio = ((double)uf->initialWidth) / uf->initialHeight;
+        else
+            /* full rotated image / manually entered crop */
+            uf->conf->aspectRatio = ((double)uf->conf->CropX2 - uf->conf->CropX1)
+                                    / (uf->conf->CropY2 - uf->conf->CropY1);
+    } else {
+        /* given aspectRatio */
+        int cropWidth = uf->conf->CropX2 - uf->conf->CropX1;
+        int cropHeight = uf->conf->CropY2 - uf->conf->CropY1;
+
+        if (cropWidth != (int)floor(cropHeight * uf->conf->aspectRatio + 0.5)) {
+            /* aspectRatio does not match the crop area - shrink the area */
+
+            if ((double)cropWidth / cropHeight > uf->conf->aspectRatio) {
+                cropWidth = floor(cropHeight * uf->conf->aspectRatio + 0.5);
+                uf->conf->CropX1 = (uf->conf->CropX1 + uf->conf->CropX2 - cropWidth) / 2;
+                uf->conf->CropX2 = uf->conf->CropX1 + cropWidth;
+            } else {
+                cropHeight = floor(cropWidth / uf->conf->aspectRatio + 0.5);
+                uf->conf->CropY1 = (uf->conf->CropY1 + uf->conf->CropY2 - cropHeight) / 2;
+                uf->conf->CropY2 = uf->conf->CropY1 + cropHeight;
+            }
+        }
+    }
 }
 
 /* Get scaled crop coordinates in final image coordinates */
@@ -433,7 +466,8 @@ int ufraw_config(ufraw_data *uf, conf_data *rc, conf_data *conf, conf_data *cmd)
         if (ufraw_exif_read_input(uf) != UFRAW_SUCCESS) {
             ufraw_message(UFRAW_SET_LOG, "Error reading EXIF data from %s\n",
                           uf->filename);
-        } else {
+            // If exiv2 fails to read the EXIF data, use the EXIF tags read
+            // by dcraw.
             g_strlcpy(uf->conf->exifSource, "DCRaw", max_name);
             uf->conf->iso_speed = raw->iso_speed;
             g_snprintf(uf->conf->isoText, max_name, "%d",
@@ -455,7 +489,16 @@ int ufraw_config(ufraw_data *uf, conf_data *rc, conf_data *conf, conf_data *cmd)
     }
     ufraw_image_set_data(uf->conf->ufobject, uf);
 #ifdef HAVE_LENSFUN
-    ufraw_lensfun_init(ufgroup_element(uf->conf->ufobject, ufLensfun));
+    // Do not reset lensfun settings while loading ID.
+    UFBoolean reset = !uf->LoadingID;
+    if (conf != NULL && conf->version > 0 && conf->ufobject != NULL) {
+        UFObject *conf_lensfun_auto = ufgroup_element(conf->ufobject,
+                                      ufLensfunAuto);
+        // Do not reset lensfun settings from conf file.
+        if (ufstring_is_equal(conf_lensfun_auto, "no"))
+            reset = FALSE;
+    }
+    ufraw_lensfun_init(ufgroup_element(uf->conf->ufobject, ufLensfun), reset);
 #endif
 
     char *absname = uf_file_set_absolute(uf->filename);
@@ -498,8 +541,8 @@ int ufraw_config(ufraw_data *uf, conf_data *rc, conf_data *conf, conf_data *cmd)
     /* POSIX.1c version of ctime_r() */
     ctime_r(&raw->timestamp, uf->conf->timestampText);
 #endif
-    if (uf->conf->timestampText[strlen(uf->conf->timestampText)-1] == '\n')
-        uf->conf->timestampText[strlen(uf->conf->timestampText)-1] = '\0';
+    if (uf->conf->timestampText[strlen(uf->conf->timestampText) - 1] == '\n')
+        uf->conf->timestampText[strlen(uf->conf->timestampText) - 1] = '\0';
 
     uf->conf->timestamp = raw->timestamp;
 
@@ -680,6 +723,17 @@ int ufraw_load_raw(ufraw_data *uf)
     return UFRAW_SUCCESS;
 }
 
+/* Free any darkframe associated with conf */
+void ufraw_close_darkframe(conf_data *conf)
+{
+    if (conf && conf->darkframe != NULL) {
+        ufraw_close(conf->darkframe);
+        g_free(conf->darkframe);
+        conf->darkframe = NULL;
+        conf->darkframeFile[0] = '\0';
+    }
+}
+
 void ufraw_close(ufraw_data *uf)
 {
     dcraw_close(uf->raw);
@@ -699,10 +753,6 @@ void ufraw_close(ufraw_data *uf)
     lf_modifier_destroy(uf->TCAmodifier);
     lf_modifier_destroy(uf->modifier);
 #endif
-    if (uf->conf->darkframe != NULL) {
-        ufraw_close(uf->conf->darkframe);
-        g_free(uf->conf->darkframe);
-    }
     ufobject_delete(uf->conf->ufobject);
     g_free(uf->conf);
     ufraw_message_reset(uf);
@@ -729,8 +779,7 @@ UFRectangle ufraw_image_get_subarea_rectangle(ufraw_image_data *img,
 
 /* Return the subarea index given some X,Y image coordinates.
  */
-unsigned ufraw_img_get_subarea_idx(
-    ufraw_image_data *img, int x, int y)
+unsigned ufraw_img_get_subarea_idx(ufraw_image_data *img, int x, int y)
 {
     int saw = (img->width + 3) / 4;
     int sah = (img->height + 7) / 8;
@@ -792,6 +841,13 @@ int ufraw_convert_image(ufraw_data *uf)
         *img = *img2;
         img2->buffer = NULL;
     }
+    if (uf->conf->autoCrop && !uf->LoadingID) {
+        ufraw_get_image_dimensions(uf);
+        uf->conf->CropX1 = (uf->rotatedWidth - uf->autoCropWidth) / 2;
+        uf->conf->CropX2 = uf->conf->CropX1 + uf->autoCropWidth;
+        uf->conf->CropY1 = (uf->rotatedHeight - uf->autoCropHeight) / 2;
+        uf->conf->CropY2 = uf->conf->CropY1 + uf->autoCropHeight;
+    }
     return UFRAW_SUCCESS;
 }
 
@@ -849,7 +905,7 @@ static void ufraw_convert_image_transform(ufraw_data *uf, ufraw_image_data *img,
             gint32 yy = (gint32)srcY;
             // TODO: better handling of the borders.
             if (xx < 0 || yy < 0 || xx + 1 >= img->width || yy + 1 >= img->height) {
-                for (c = 0; c < 3; c++)
+                for (c = 0; c < uf->colors; c++)
                     cur[c] = 0;
                 continue;
             }
@@ -860,7 +916,7 @@ static void ufraw_convert_image_transform(ufraw_data *uf, ufraw_image_data *img,
             for (c = 0; c < uf->colors; c++)
                 cur[c] = ((4096 - dy) * ((4096 - dx) * src[0][c] + dx * src[1][c])
                           + dy * ((4096 - dx) * src[img->width][c]
-                                  + (dx) * src[img->width+1][c])) >> 24;
+                                  + (dx) * src[img->width + 1][c])) >> 24;
         }
     }
 }
@@ -882,7 +938,8 @@ static void ufraw_convert_image_transform(ufraw_data *uf, ufraw_image_data *img,
  * -	use uf->rgbMax (check, must be about 64k)
  */
 static void ufraw_shave_hotpixels(ufraw_data *uf, dcraw_image_type *img,
-                                  int width, int height, int colors, unsigned rgbMax)
+                                  int width, int height, int colors,
+                                  unsigned rgbMax)
 {
     int w, h, c, i, count;
     unsigned delta, t, v, hi;
@@ -894,10 +951,10 @@ static void ufraw_shave_hotpixels(ufraw_data *uf, dcraw_image_type *img,
     delta = rgbMax / (uf->conf->hotpixel + 1.0);
     count = 0;
 #ifdef _OPENMP
-#pragma omp parallel for schedule(static) default(none) \
-  shared(uf,img,width,height,colors,rgbMax,delta) \
-  reduction(+:count) \
-  private(h,p,w,c,t,v,hi,i)
+    #pragma omp parallel for schedule(static) default(none) \
+    shared(uf,img,width,height,colors,rgbMax,delta) \
+reduction(+:count) \
+    private(h,p,w,c,t,v,hi,i)
 #endif
     for (h = 1; h < height - 1; ++h) {
         p = img + 1 + h * width;
@@ -942,7 +999,7 @@ static void ufraw_shave_hotpixels(ufraw_data *uf, dcraw_image_type *img,
 }
 
 static void ufraw_despeckle_line(guint16 *base, int step, int size, int window,
-                                 double decay, int colors, int c)
+        double decay, int colors, int c)
 {
     unsigned lum[size];
     int i, j, start, end, next, v, cold, hot, coldj, hotj, fix;
@@ -1022,7 +1079,7 @@ void ufraw_despeckle(ufraw_data *uf, UFRawPhase phase)
             if (pass >= passes[c])
                 continue;
 #ifdef _OPENMP
-#pragma omp parallel for default(shared) private(i,base)
+            #pragma omp parallel for default(shared) private(i,base)
 #endif
             for (i = 0; i < img->height; ++i) {
                 base = (guint16 *)img->buffer + i * rowstride;
@@ -1030,7 +1087,7 @@ void ufraw_despeckle(ufraw_data *uf, UFRawPhase phase)
                                      decay[c], colors, c);
             }
 #ifdef _OPENMP
-#pragma omp parallel for default(shared) private(i,base)
+            #pragma omp parallel for default(shared) private(i,base)
 #endif
             for (i = 0; i < img->width; ++i) {
                 base = (guint16 *)img->buffer + i * depth;
@@ -1055,6 +1112,13 @@ static gboolean ufraw_despeckle_active(ufraw_data *uf)
 
 static int ufraw_calculate_scale(ufraw_data *uf)
 {
+    /* In the first call to ufraw_calculate_scale() the crop coordinates
+     * are not set. They cannot be set, since uf->rotatedHeight/Width are
+     * only calculated later in ufraw_convert_prepare_transform_buffer().
+     * Therefore, if size > 0, scale = 1 will be returned.
+     * Since the first call is from ufraw_convert_prepare_first_buffer(),
+     * this is not a real issue. There should always be a second call to
+     * this function before the actual buffer allocation. */
     dcraw_data *raw = uf->raw;
     int scale = 1;
 
@@ -1065,8 +1129,8 @@ static int ufraw_calculate_scale(ufraw_data *uf)
         scale = uf->conf->shrink * MIN(raw->pixel_aspect, 1 / raw->pixel_aspect);
     } else if (uf->conf->interpolation == half_interpolation) {
         scale = 2;
-        /* Wanted size is smaller than raw size (size is after a raw->shrink)
-         * (assuming there are filters). */
+    /* Wanted size is smaller than raw size (size is after a raw->shrink)
+     * (assuming there are filters). */
     } else if (uf->conf->size > 0 && uf->HaveFilters) {
         int cropHeight = uf->conf->CropY2 - uf->conf->CropY1;
         int cropWidth = uf->conf->CropX2 - uf->conf->CropX1;
@@ -1096,9 +1160,15 @@ static void ufraw_convertshrink(ufraw_data *uf, dcraw_image_data *final)
                            scale * MAX(final->height, final->width) / uf->conf->shrink);
     }
     if (uf->conf->size > 0) {
-        int cropHeight = uf->conf->CropY2 - uf->conf->CropY1;
-        int cropWidth = uf->conf->CropX2 - uf->conf->CropX1;
-        int cropSize = MAX(cropHeight, cropWidth);
+        int finalSize = scale * MAX(final->height, final->width);
+        int cropSize;
+        if (uf->conf->CropX1 == -1) {
+            cropSize = finalSize;
+        } else {
+            int cropHeight = uf->conf->CropY2 - uf->conf->CropY1;
+            int cropWidth = uf->conf->CropX2 - uf->conf->CropX1;
+            cropSize = MAX(cropHeight, cropWidth);
+        }
         // cropSize needs to be a integer multiplier of scale
         cropSize = cropSize / scale * scale;
         if (uf->conf->size > cropSize) {
@@ -1108,7 +1178,6 @@ static void ufraw_convertshrink(ufraw_data *uf, dcraw_image_data *final)
             /* uf->conf->size holds the size of the cropped image.
              * We need to calculate from it the desired size of
              * the uncropped image. */
-            int finalSize = scale * MAX(final->height, final->width);
             dcraw_image_resize(final, uf->conf->size * finalSize / cropSize);
         }
     }
@@ -1168,7 +1237,7 @@ static void ufraw_convert_image_first(ufraw_data *uf, UFRawPhase phase)
     raw->raw.image = rawimage;
     dcraw_flip_image(&final, uf->conf->orientation);
 
-    // The 'out' image contains the predictated image dimensions.
+    // The 'out' image contains the predicted image dimensions.
     // We want to be sure that our predictions were correct.
     if (out->height != final.height) {
         g_warning("ufraw_convert_image_first: height mismatch %d!=%d",
@@ -1202,9 +1271,9 @@ static void ufraw_convert_reverse_wb(ufraw_data *uf, UFRawPhase phase)
         mul[i] = (guint64)0x10000 * 0x10000 / uf->developer->rgbWB[i];
     size = img->height * img->width;
 #ifdef _OPENMP
-#pragma omp parallel for schedule(static) default(none) \
-  shared(uf,phase,img,mul,size) \
-  private(i,p16,c,px)
+    #pragma omp parallel for schedule(static) default(none) \
+    shared(uf,phase,img,mul,size) \
+    private(i,p16,c,px)
 #endif
     for (i = 0; i < size; ++i) {
         p16 = (guint16 *)&img->buffer[i * img->depth];
@@ -1220,14 +1289,15 @@ static void ufraw_convert_reverse_wb(ufraw_data *uf, UFRawPhase phase)
 #ifdef HAVE_LENSFUN
 /* Apply TCA */
 static void ufraw_convert_image_tca(ufraw_data *uf, ufraw_image_data *img,
-                                    ufraw_image_data *outimg, UFRectangle *area)
+                                    ufraw_image_data *outimg,
+                                    UFRectangle *area)
 {
     if (uf->TCAmodifier == NULL)
         return;
     int y;
 #ifdef _OPENMP
-#pragma omp parallel for schedule(static) default(none) \
-  shared(uf,img,outimg,area)
+    #pragma omp parallel for schedule(static) default(none) \
+    shared(uf,img,outimg,area)
 #endif
     for (y = area->y; y < area->y + area->height; y++) {
         guint16 *dst = (guint16*)(outimg->buffer + y * outimg->rowstride +
@@ -1261,7 +1331,7 @@ static void ufraw_convert_image_tca(ufraw_data *uf, ufraw_image_data *img,
                 dst[c] = ((4096 - dy) * ((4096 - dx) * lf_src[0][c]
                                          + dx * lf_src[1][c])
                           + dy * ((4096 - dx) * lf_src[img->width][c]
-                                  + (dx) * lf_src[img->width+1][c])) >> 24;
+                                  + (dx) * lf_src[img->width + 1][c])) >> 24;
             }
             modcoord -= 2;
             // Green channels are intact
@@ -1272,7 +1342,8 @@ static void ufraw_convert_image_tca(ufraw_data *uf, ufraw_image_data *img,
 }
 #endif // HAVE_LENSFUN
 
-static void ufraw_convert_import_buffer(ufraw_data *uf, UFRawPhase phase, dcraw_image_data *dcimg)
+static void ufraw_convert_import_buffer(ufraw_data *uf, UFRawPhase phase,
+                                        dcraw_image_data *dcimg)
 {
     ufraw_image_data *img = &uf->Images[phase];
 
@@ -1285,7 +1356,7 @@ static void ufraw_convert_import_buffer(ufraw_data *uf, UFRawPhase phase, dcraw_
 }
 
 static void ufraw_image_init(ufraw_image_data *img,
-                             int width, int height, int bitdepth)
+        int width, int height, int bitdepth)
 {
     if (img->height == height && img->width == width &&
             img->depth == bitdepth && img->buffer != NULL)
@@ -1314,9 +1385,15 @@ static void ufraw_convert_prepare_first_buffer(ufraw_data *uf,
         img->height = img->height * scale / uf->conf->shrink;
     }
     if (uf->conf->size > 0) {
-        int cropHeight = uf->conf->CropY2 - uf->conf->CropY1;
-        int cropWidth = uf->conf->CropX2 - uf->conf->CropX1;
-        int cropSize = MAX(cropHeight, cropWidth);
+        int finalSize = scale * MAX(img->height, img->width);
+        int cropSize;
+        if (uf->conf->CropX1 == -1) {
+            cropSize = finalSize;
+        } else {
+            int cropHeight = uf->conf->CropY2 - uf->conf->CropY1;
+            int cropWidth = uf->conf->CropX2 - uf->conf->CropX1;
+            cropSize = MAX(cropHeight, cropWidth);
+        }
         // cropSize needs to be a integer multiplier of scale
         cropSize = cropSize / scale * scale;
         if (uf->conf->size > cropSize) {
@@ -1326,7 +1403,6 @@ static void ufraw_convert_prepare_first_buffer(ufraw_data *uf,
             /* uf->conf->size holds the size of the cropped image.
              * We need to calculate from it the desired size of
              * the uncropped image. */
-            int finalSize = scale * MAX(img->height, img->width);
             int mul = uf->conf->size * finalSize / cropSize;
             int div = MAX(img->height, img->width);
             img->height = img->height * mul / div;
@@ -1337,7 +1413,8 @@ static void ufraw_convert_prepare_first_buffer(ufraw_data *uf,
 
 #ifdef HAVE_LENSFUN
 void ufraw_convert_prepare_transform(ufraw_data *uf,
-                                     int width, int height, gboolean reverse, float scale);
+                                     int width, int height, gboolean reverse,
+                                     float scale);
 #endif
 
 static void ufraw_convert_prepare_transform_buffer(ufraw_data *uf,
@@ -1345,6 +1422,12 @@ static void ufraw_convert_prepare_transform_buffer(ufraw_data *uf,
 {
     const int iWidth = uf->initialWidth;
     const int iHeight = uf->initialHeight;
+
+    double aspectRatio = uf->conf->aspectRatio;
+
+    if (aspectRatio == 0)
+        aspectRatio = ((double)iWidth) / iHeight;
+
 #ifdef HAVE_LENSFUN
     ufraw_convert_prepare_transform(uf, iWidth, iHeight, TRUE, 1.0);
     if (uf->conf->rotationAngle == 0 &&
@@ -1364,12 +1447,16 @@ static void ufraw_convert_prepare_transform_buffer(ufraw_data *uf,
         uf->rotatedHeight = iHeight;
         uf->autoCropWidth = iWidth;
         uf->autoCropHeight = iHeight;
+        if ((double)uf->autoCropWidth / uf->autoCropHeight > aspectRatio)
+            uf->autoCropWidth = floor(uf->autoCropHeight * aspectRatio + 0.5);
+        else
+            uf->autoCropHeight = floor(uf->autoCropWidth / aspectRatio + 0.5);
+
         return;
     }
     const double sine = sin(uf->conf->rotationAngle * 2 * M_PI / 360);
     const double cosine = cos(uf->conf->rotationAngle * 2 * M_PI / 360);
-    const float aspectRatio = (float)(uf->conf->CropX2 - uf->conf->CropX1) /
-                              (uf->conf->CropY2 - uf->conf->CropY1);
+
     const float midX = iWidth / 2.0 - 0.5;
     const float midY = iHeight / 2.0 - 0.5;
 #ifdef HAVE_LENSFUN
@@ -1416,14 +1503,17 @@ static void ufraw_convert_prepare_transform_buffer(ufraw_data *uf,
     }
     float scale = sqrt((iWidth - 1) * (iHeight - 1) / area);
     // Do not allow increasing canvas size by more than a factor of 2
-    uf->rotatedWidth = MIN(ceil(2 * maxX) * scale, 2 * iWidth);
-    uf->rotatedHeight = MIN(ceil(2 * maxY) * scale, 2 * iHeight);
-    if (minX / minY > aspectRatio)
-        minX = minY * aspectRatio;
-    else
-        minY = minX / aspectRatio;
+    uf->rotatedWidth = MIN(ceil(2 * maxX + 1.0) * scale, 2 * iWidth);
+    uf->rotatedHeight = MIN(ceil(2 * maxY + 1.0) * scale, 2 * iHeight);
+
     uf->autoCropWidth = MIN(floor(2 * minX) * scale, 2 * iWidth);
     uf->autoCropHeight = MIN(floor(2 * minY) * scale, 2 * iHeight);
+
+    if ((double)uf->autoCropWidth / uf->autoCropHeight > aspectRatio)
+        uf->autoCropWidth = floor(uf->autoCropHeight * aspectRatio + 0.5);
+    else
+        uf->autoCropHeight = floor(uf->autoCropWidth / aspectRatio + 0.5);
+
     int newWidth = uf->rotatedWidth * width / iWidth;
     int newHeight = uf->rotatedHeight * height / iHeight;
     ufraw_image_init(img, newWidth, newHeight, 8);
@@ -1446,8 +1536,8 @@ static void ufraw_convert_prepare_buffers(ufraw_data *uf, UFRawPhase phase)
     int width = 0, height = 0;
     if (phase > ufraw_first_phase) {
         ufraw_convert_prepare_buffers(uf, phase - 1);
-        width = uf->Images[phase-1].width;
-        height = uf->Images[phase-1].height;
+        width = uf->Images[phase - 1].width;
+        height = uf->Images[phase - 1].height;
     }
     switch (phase) {
         case ufraw_raw_phase:
@@ -1484,7 +1574,7 @@ static void ufraw_convert_prepare_buffers(ufraw_data *uf, UFRawPhase phase)
  * constants by variables may turn off some compiler optimizations.
  */
 static void ufraw_image_format(int *colors, int *bytes, ufraw_image_data *img,
-                               const char *formats, const char *caller)
+        const char *formats, const char *caller)
 {
     int b, c;
 
@@ -1517,7 +1607,7 @@ static void ufraw_image_format(int *colors, int *bytes, ufraw_image_data *img,
 }
 
 ufraw_image_data *ufraw_get_image(ufraw_data *uf, UFRawPhase phase,
-                                  gboolean bufferok)
+        gboolean bufferok)
 {
     ufraw_convert_prepare_buffers(uf, phase);
     // Find the closest phase that is actually rendered:
@@ -1639,7 +1729,7 @@ ufraw_image_data *ufraw_convert_image_area(ufraw_data *uf, unsigned saidx,
     }
 
 #ifdef _OPENMP
-#pragma omp critical
+    #pragma omp critical
 #endif
     // Mark the subarea as valid
     out->valid |= (1 << saidx);
@@ -2186,7 +2276,7 @@ void ufraw_auto_curve(ufraw_data *uf)
         for (c = 0, p = 0; c < 3; c++) p = MAX(p, p16[c]);
         stop += uf->RawCount * pow(decay, i) / norm;
         /* Skip adding point if slope is too big (more than 4) */
-        if (j > 0 && p - curve->m_anchors[j-1].x * 0x10000 < (i + 1 - j) * 0x04000 / steps)
+        if (j > 0 && p - curve->m_anchors[j - 1].x * 0x10000 < (i + 1 - j) * 0x04000 / steps)
             continue;
         curve->m_anchors[j].x = (double)p / 0x10000;
         curve->m_anchors[j].y = (double)i / steps;
@@ -2199,10 +2289,10 @@ void ufraw_auto_curve(ufraw_data *uf)
         /* The last point can be up to twice the height of a linear
          * interpolation of the last two points */
         if (j > 1) {
-            curve->m_anchors[j].y = curve->m_anchors[j-1].y +
-                                    2 * (1.0 - curve->m_anchors[j-1].x) *
-                                    (curve->m_anchors[j-1].y - curve->m_anchors[j-2].y) /
-                                    (curve->m_anchors[j-1].x - curve->m_anchors[j-2].x);
+            curve->m_anchors[j].y = curve->m_anchors[j - 1].y +
+                                    2 * (1.0 - curve->m_anchors[j - 1].x) *
+                                    (curve->m_anchors[j - 1].y - curve->m_anchors[j - 2].y) /
+                                    (curve->m_anchors[j - 1].x - curve->m_anchors[j - 2].x);
             if (curve->m_anchors[j].y > 1.0) curve->m_anchors[j].y = 1.0;
         } else {
             curve->m_anchors[j].y = 1.0;
