@@ -2,7 +2,7 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * ufraw_writer.c - functions to output image files in different formats.
- * Copyright 2004-2011 by Udi Fuchs
+ * Copyright 2004-2013 by Udi Fuchs
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -70,7 +70,7 @@ static int ppm_row_writer(ufraw_data *uf, void *volatile out, void *pixbuf,
             pixbuf16[i] = g_htons(pixbuf16[i]);
     }
     for (i = 0; i < height; i++) {
-        if ((int)fwrite(pixbuf + i * width*(bitDepth > 8 ? 6 : 3), rowStride, 1, out) < 1) {
+        if ((int)fwrite(pixbuf + i * width * (bitDepth > 8 ? 6 : 3), rowStride, 1, out) < 1) {
             ufraw_set_error(uf, _("Error creating file '%s'."),
                             uf->conf->outputFilename);
             ufraw_set_error(uf, g_strerror(errno));
@@ -214,13 +214,13 @@ void ufraw_write_image_data(
     for (row0 = 0; row0 < Crop->height; row0 += DEVELOP_BATCH) {
         progress(PROGRESS_SAVE, DEVELOP_BATCH);
 #ifdef _OPENMP
-#pragma omp parallel for default(shared) private(row)
+        #pragma omp parallel for default(shared) private(row)
 #endif
         for (row = 0; row < DEVELOP_BATCH; row++) {
             if (row + row0 >= Crop->height)
                 continue;
             guint8 *rowbuf = &pixbuf8[row * Crop->width * 3 * byteDepth];
-            develop(rowbuf, rawImage[(Crop->y+row+row0)*rowStride+Crop->x],
+            develop(rowbuf, rawImage[(Crop->y + row + row0)*rowStride + Crop->x],
                     uf->developer, bitDepth, Crop->width);
             if (grayscaleMode)
                 grayscale_buffer(rowbuf, Crop->width, bitDepth);
@@ -254,6 +254,13 @@ int ufraw_write_image(ufraw_data *uf)
         }
     }
     if (uf->conf->createID == only_id) {
+        if (uf->conf->autoCrop && !uf->LoadingID) {
+            ufraw_get_image_dimensions(uf);
+            uf->conf->CropX1 = (uf->rotatedWidth - uf->autoCropWidth) / 2;
+            uf->conf->CropX2 = uf->conf->CropX1 + uf->autoCropWidth;
+            uf->conf->CropY1 = (uf->rotatedHeight - uf->autoCropHeight) / 2;
+            uf->conf->CropY2 = uf->conf->CropY1 + uf->autoCropHeight;
+        }
         int status = conf_save(uf->conf, confFilename, NULL);
         g_free(confFilename);
         return status;
@@ -296,7 +303,13 @@ int ufraw_write_image(ufraw_data *uf)
             int status = 0;
             char *filename =
                 uf_win32_locale_filename_from_utf8(uf->conf->outputFilename);
-            fits_create_file(&fitsFile, filename, &status);
+            if (strcmp(filename, "-") != 0)
+                // Use fits_create_diskfile() to allow more characters in
+                // filenames.
+                fits_create_diskfile(&fitsFile, filename, &status);
+            else
+                // fits_create_file() can write to stdout.
+                fits_create_file(&fitsFile, filename, &status);
             uf_win32_locale_filename_free(filename);
             if (status) {
                 ufraw_set_error(uf, _("Error creating file '%s'."),
@@ -608,11 +621,11 @@ int ufraw_write_image(ufraw_data *uf)
             progress(PROGRESS_SAVE, 1);
             for (i = 0; i < Crop.width; i++) {
                 offset = row * Crop.width + i;
-                develop_linear(rawImage[(Crop.y+row)*rowStride+Crop.x+i], pixbuf16,
+                develop_linear(rawImage[(Crop.y + row)*rowStride + Crop.x + i], pixbuf16,
                                uf->developer);
                 int c;
                 for (c = 0; c < 3; c++) {
-                    sum[c] += image[c*dim + offset] = pixbuf16[c];
+                    sum[c] += image[c * dim + offset] = pixbuf16[c];
                     max[c] = MAX(pixbuf16[c], max[c]);
                     min[c] = MIN(pixbuf16[c], min[c]);
                 }
@@ -812,7 +825,12 @@ static void PNGwriteRawProfile(png_struct *ping,
     *dp++ = '\n';
     *dp = '\0';
 
+#if ((PNG_LIBPNG_VER_MAJOR > 1) || ((PNG_LIBPNG_VER_MAJOR == 1) && \
+(PNG_LIBPNG_VER_MINOR > 2))) && (defined(INT_MAX) && (INT_MAX > 0x7ffffffeL))
+    g_snprintf(dp, allocated_length - strlen(text[0].text), "%8u ", length);
+#else
     g_snprintf(dp, allocated_length - strlen(text[0].text), "%8lu ", length);
+#endif
 
     dp += 8;
 
